import pandas as pd

from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.metadata.SignalMetadata import SignalMetadata


def get_lhc_context(start_time_date, duration=[(1, 's'), (2, 's')]):
    t_start, t_end = Time.get_query_period_in_unix_time(start_time_date=start_time_date, duration_date=duration)

    # Fill number
    pm_fill_num_df = PmDbRequest.query_context(t_start=t_start, t_end=t_start + int(2e9), system='BLM',
                                               className='BLMLHC', source='HC.BLM.SR6.C', parameters=["pmFillNum"])

    # Beam energy and intensity
    lhc_parameters_df = PmDbRequest.query_context(t_start=t_start, t_end=t_start + int(2e9), system='LHC',
                                                  className='CISX', source='CISX.CCR.LHC.A',
                                                  parameters=["OVERALL_ENERGY", "OVERALL_INTENSITY_1",
                                                              "OVERALL_INTENSITY_2"])
    lhc_parameters_df['OVERALL_ENERGY'] *= 1.2  # magic number

    # Beam Mode

    lhc_bm_df = PmDbRequest.query_context(t_start=t_start, t_end=t_start + int(2e9), system='LHC', className='CISX',
                                          source='CISX.CCR.LHC.GA', parameters=["BEAM_MODE"])
    beam_mode_df = SignalMetadata.get_beam_mode_details()
    lhc_bm_df['BEAM_MODE'] = beam_mode_df[beam_mode_df['Mode'] == lhc_bm_df['BEAM_MODE'].values[0]]['Name'].values[0]

    # Abort gap
    abort_gap_b1_df = PmDbRequest.query_context(t_start=t_start, t_end=t_start + int(5e9), system='LBDS',
                                                className='BSRA', source='LHC.BSRA.US45.B1',
                                                parameters=['aGXpocTotalIntensity', 'aGXpocTotalMaxIntensity'])
    abort_gap_b1_df.columns = ['%sB1' % col for col in abort_gap_b1_df.columns]


    abort_gap_b2_df = PmDbRequest.query_context(t_start=t_start, t_end=t_start + int(5e9), system='LBDS',
                                                className='BSRA', source='LHC.BSRA.US45.B2',
                                                parameters=['aGXpocTotalIntensity', 'aGXpocTotalMaxIntensity'])
    abort_gap_b2_df.columns = ['%sB2' % col for col in abort_gap_b2_df.columns]

    # Summary
    lhc_data_df = pd.concat([pm_fill_num_df, lhc_parameters_df, lhc_bm_df], axis=1)
    lhc_data_df['timestamp_blm'] = lhc_data_df.index
    lhc_data_df.index = pm_fill_num_df['pmFillNum']
    lhc_data_df.drop(columns=['pmFillNum'], inplace=True)

    abort_gap_df = pd.concat([abort_gap_b1_df, abort_gap_b2_df], axis=1)
    abort_gap_df['timestamp_abort_gap'] = abort_gap_df.index
    abort_gap_df.index = pm_fill_num_df['pmFillNum']

    return pd.concat([lhc_data_df, abort_gap_df], axis=1)
